From a64514df4fcd00413580d16e6418223ae0eb6ae6 Mon Sep 17 00:00:00 2001
From: "Peder O. Klingenberg" <peder@klingenberg.no>
Date: Sat, 14 Mar 2015 23:00:16 +0100
Subject: [PATCH] Extend process-lines to allow exit status handling

* subr.el (process-lines-handling-status): Extension of the old
process-lines, with more flexible handling of the exit status.
(process-lines): Old API implemented using the new function.
(process-lines-ignore-status): Another use of the new function -
return the output lines regardless of the exit status.

Fixes: Bug#1321
---
 lisp/ChangeLog |  8 ++++++++
 lisp/subr.el   | 24 ++++++++++++++++++++----
 2 files changed, 28 insertions(+), 4 deletions(-)

diff --git a/lisp/ChangeLog b/lisp/ChangeLog
index d393190..a93a306 100644
--- a/lisp/ChangeLog
+++ b/lisp/ChangeLog
@@ -1,3 +1,11 @@
+2015-03-14  Peder O. Klingenberg  <peder@klingenberg.no>
+
+	* subr.el (process-lines-handling-status): Extension of the old
+	process-lines, with more flexible handling of the exit status.
+	(process-lines): Old API implemented using the new function.
+	(process-lines-ignore-status): Another use of the new function -
+	return the output lines regardless of the exit status.
+
 2015-03-13  Kevin Ryde  <user42_kevin@yahoo.com.au>
 
 	info-look fixes for Texinfo 5
diff --git a/lisp/subr.el b/lisp/subr.el
index deadca6..ec471f0 100644
--- a/lisp/subr.el
+++ b/lisp/subr.el
@@ -1901,13 +1901,19 @@ and the file name is displayed in the echo area."
 
 ;;;; Process stuff.
 
-(defun process-lines (program &rest args)
+(defun process-lines-handling-status (program status-handler &rest args)
   "Execute PROGRAM with ARGS, returning its output as a list of lines.
-Signal an error if the program returns with a non-zero exit status."
+If STATUS-HANDLER is non-NIL, it must be a function with one
+argument, which will be called with the exit status of the
+program before the output is collected.  If STATUS-HANDLER is
+NIL, an error is signalled if the program returns with a non-zero
+exit status."
   (with-temp-buffer
     (let ((status (apply 'call-process program nil (current-buffer) nil args)))
-      (unless (eq status 0)
-	(error "%s exited with status %s" program status))
+      (if status-handler
+	  (funcall status-handler status)
+	(unless (eq status 0)
+	  (error "%s exited with status %s" program status)))
       (goto-char (point-min))
       (let (lines)
 	(while (not (eobp))
@@ -1918,6 +1924,16 @@ Signal an error if the program returns with a non-zero exit status."
 	  (forward-line 1))
 	(nreverse lines)))))
 
+(defun process-lines (program &rest args)
+  "Execute PROGRAM with ARGS, returning its output as a list of lines.
+Signal an error if the program returns with a non-zero exit status."
+  (apply #'process-lines-handling-status program nil args))
+
+(defun process-lines-ignore-status (program &rest args)
+  "Execute PROGRAM with ARGS, returning its output as a list of lines.
+The exit status of the program is ignored."
+  (apply #'process-lines-handling-status program #'identity args))
+
 (defun process-live-p (process)
   "Returns non-nil if PROCESS is alive.
 A process is considered alive if its status is `run', `open',
-- 
1.9.5 (Apple Git-50.3)

